% Structured
% ts: four columns --- z, n, #exits, #entrants

global zmax v A B C D cnt gx ge sigma a beta ve emax sv pi gxx T Q nn AA BB nstates Mx Me fp

setupstart = cputime

zmax = 45; %the distribution of changes in z is estimated externally. wlog, zmin = 1
gmax = 3;
emax = max(max(xs(:,4,:))); % the max number of entrants ever observed
nmax = max(max(xs(:,2,:)+emax*sign(xs(:,4,:)))); % estimate of the max possible number of incumbents
% if in a given state 1 entrant is observed, theoretically emax could be
% observed as well, and so nmax is equal to emax+the largest n at which there is entry


cexits = zeros(zmax, nmax+1, gmax);  %count the number of exits in each possible state, including n=0
cstates = zeros(zmax, nmax+1, gmax); %count the number of times each state is reached
centr = zeros(zmax, nmax+1, gmax); %count the number of entrants in each state

T = size(xs, 1);
Q = size(xs, 3);
for q = 1:Q
    ts = xs(:,:,q);
    for t = 2:T-1
        cstates(ts(t,1),ts(t,2)+1, ts(t,1)-ts(t-1,1)+2) = cstates(ts(t,1),ts(t,2)+1, ts(t,1)-ts(t-1,1)+2) + 1;
        cexits(ts(t,1),ts(t,2)+1, ts(t,1)-ts(t-1,1)+2) = cexits(ts(t,1),ts(t,2)+1, ts(t,1)-ts(t-1,1)+2) + ts(t,3);
        centr(ts(t,1),ts(t,2)+1, ts(t,1)-ts(t-1,1)+2) = centr(ts(t,1),ts(t,2)+1, ts(t,1)-ts(t-1,1)+2) + ts(t,4);
    end;
end;

avx = zeros(zmax, nmax+1, gmax); % estimating probabilities of exit

for z = 1:zmax
    for n = 1:nmax
        for g = 1:gmax
            if cstates(z,n+1,g) > 0
                avx(z,n+1,g) = cexits(z,n+1,g)/(n*cstates(z,n+1,g));
            else
                avx(z,n+1,g) = avx(z,n,g);
            end
        end
    end
end

ave = zeros(zmax, nmax+1, g); % estimating probabilities of entry

for z = 1:zmax
    for nn = 1:nmax
        n = nmax-nn;
        for g = 1:gmax
            if cstates(z,n+1,g) > 0
                ave(z,n+1,g) = centr(z,n+1,g)/(emax*cstates(z,n+1,g));
            else
                ave(z,n+1,g) = ave(z,n+2,g);
            end
        end
    end
end    

nstates=zeros(zmax,nmax+1,g);
v=[];

for z=1:zmax
    for n=0:nmax
        for g=1:gmax
            v=[v;z n g];
            nstates(z,n+1,g)=size(v,1);
        end
    end
end

sv = size(v,1);

% pbentrR(a,b,c) is the probability (from the point of view of an econometrician)
% that there are (a-1) entrants given that there are (b-1) incumbents
% and z=c, assuming that ave are the true entry probabilities

pbentrR=zeros(emax+1, nmax+1, zmax, gmax);
for n=0:nmax-1
    for z=1:zmax
        for g=1:gmax
            F=ave(z,n+1,g);
            for j=0:emax
                pbentrR(j+1,n+1,z,g)=nchoosek(emax,j)*F^j*(1-F)^(emax-j);
            end
        end
    end
end

pbentrR(1,nmax+1,:,:)=ones(zmax,gmax); %if there are nmax guys, nobody can enter

% bR(a,b,c) is the probability that (a-1) firms exit given that
% there are (b-1) firms and z=c, from the point of view of an econometrician,
% assuming that avx are the true exit probabilities

bR=zeros(nmax+1, nmax+1, zmax, gmax);
for i=0:nmax
    for j=1:nmax
        for z=1:zmax
            for g=1:gmax
                if i<=j
                    F=avx(z,j+1,g);
                    bR(i+1, j+1,z,g)=nchoosek(j,i)*F^(i)*(1-F)^(j-i);
                end
            end
        end
    end
end

bR(1,1,:,:)=ones(zmax,gmax); %if there are 0 guys, 0 of them exit with probability 1

TMe=zeros(sv);
TMx=zeros(sv);

for n=0:nmax
    for z=1:zmax
        for g=1:gmax
            for dx=0:n
                for de=0:emax
                    dzl=min(zmax-z,max(max(-1,(g-3)),1-z));
                    dzh=max(1-z,min(min(1,(g-1)),zmax-z));
                    for dz=dzl:dzh
                        nnew=min(n+de-dx, nmax);
                        znew=z+dz;
                        gnew=dz+2;
                        switch dzh-dzl
                            case 0
                                pdz = 1;
                            case 1
                                pdz = .25 + (gnew==g)/2;
                            case 2
                                pdz = .25 + (dz==0)/4;
                            otherwise
                                disp('Something is wrong with "dzh-dzl"');
                        end
                        TMe(nstates(z,n+1,g), nstates(znew, nnew+1, gnew)) = TMe(nstates(z,n+1,g), nstates(znew, nnew+1, gnew)) + de*pbentrR(de+1, n+1, z, g)*bR(dx+1, n+1, z, g)*pdz;         
                        TMx(nstates(z,n+1,g), nstates(znew, nnew+1, gnew)) = TMx(nstates(z,n+1,g), nstates(znew, nnew+1, gnew)) + (n-dx)*pbentrR(de+1, n+1, z, g)*bR(dx+1, n+1, z, g)*pdz;
                    end
                end
            end
        end
    end
end

Te = sum(TMe')';
Tx = sum(TMx')';

Mx = TMx./(max(.000001, Tx)*ones(1, size(Tx,1)));
Me = TMe./(max(.000001, Te)*ones(1, size(Te,1)));

pi=zeros(sv,1);
for s=1:sv
    pi(s)= profit(v(s,2), v(s,1));
end

gxx=zeros(sv,1);
for s=1:sv
    gxx(s)= avx(v(s,1), v(s,2)+1, v(s,3));
end

gx = zeros(sv,1); 
ge = zeros(sv,1);
cnt = zeros(sv,1);

for s = 1:sv
    cnt(s) = cstates(v(s,1),v(s,2)+1,v(s,3));
    if cnt(s)>0
        gx(s) = cexits(v(s,1),v(s,2)+1,v(s,3))/cnt(s);
        ge(s) = centr(v(s,1),v(s,2)+1,v(s,3))/cnt(s);
    end
end

% the part below is essentially identical to the similar part in uxs.m

beta = .9;
invmat = inv(eye(sv)-beta*Mx)*Mx;
A = invmat * pi;
B = invmat * beta * gxx;
C = Me * (pi + beta * A);
D = Me * beta * (B + gxx);

AA = A;
BB = B;
nn = v(:,2);

setupend = cputime;

nn = v(1:sv, 2);
A = AA(1:sv, 1);
B = BB(1:sv, 1);

sest = 1; % for record-keeping purposes; PR = 1

%First---simple estimator, OF=VF=0, PR = 1 

init = cputime;
options = optimset('Display','iter');
[sigma,fval,exitflag,output] = fzero(@momx, [0.01 5], options);
cc1 = output.funcCount;
ve=Me*(pi + beta*(AA + (BB+gxx)/sigma));
[a,fval,exitflag,output] = fzero(@mome, [0.01 5], options);
cc1 = cc1 + output.funcCount;

% Setup Time, Function Count, Solver Time, a, sigma, MLE/GMM, Struc/Unstruc, FP/Mat.Inv, ...
sres = [setupend - setupstart, cc1, cputime - init, a, sigma, 0, sest, 0, 0, sv, sv, Q, T, ii];

%Now - fixed point, VF=1

global vvc
init = cputime;
[sigma,fval,exitflag,output] = fzero(@momxfp, [0.01 5], options);
cc1 = output.funcCount;
ve = Me*(pi+beta*(vvc+exp(-sigma*vvc)/sigma));
[a,fval,exitflag,output] = fzero(@mome, [0.01 5], options);
cc1 = cc1 + output.funcCount;
sres = [sres; setupend - setupstart, output.funcCount, cputime - init, a, sigma, 0, sest, 1, 0, sv, sv, Q, T, ii];

% State-specific MOM, OF = 2 (in the code, momcond = 2). Both VF = 0 and VF = 1 (in the code, fp = 0 and fp = 1).
for momcond = 2:2 
    for fp = 0:1
        for grsrch = 0:0
            init = cputime;
            gridsteps = 10;
            gridlow = .1;
            gridstep = .2;
            
            valmin = Inf;
            amin = -1;
            smin = -1;
            
            if grsrch %execute grid search - disabled in the final code
                for apoint = 0:(gridsteps-1)
                    for spoint = 0:(gridsteps-1)
                        vm = momcond2([gridlow+apoint*gridstep; gridlow+spoint*gridstep]); 
                        if  vm < valmin
                            amin = gridlow+apoint*gridstep;
                            smin = gridlow+spoint*gridstep;
                            valmin = vm;
                        end
                    end
                end
            else
                amin = aguess;
                smin = sguess;
            end
            
            options = optimset('Display','iter','MaxFunEvals', 500, 'MaxIter', 500);
            fval=0;
            ef=0;
            [z,fval,ef,output]=fminsearch(@momcond2, [amin; smin], options);
            sres = [sres; setupend - setupstart, output.funcCount, cputime - init, z', momcond, sest, fp, grsrch, sv, sv, Q, T, ii];
            
            % when appropriate, save values as first-step estimators
            if (momcond==2) & (fp == 0)
                afstep = z(1,1);
                sfstep = z(2,1);
            end        
        end
    end
end

% Pseudo-MLE, both VF = 0 and VF = 1 (in the code, fp = 0 and fp = 1)

for fp = 0:1
    for grsrch = 0:0
        init = cputime;
        valmin = Inf;
        amin = -1;
        smin = -1;
        
        if grsrch %execute grid search
            for apoint = 0:(gridsteps-1)
                for spoint = 0:(gridsteps-1)
                    vm = ll([gridlow+apoint*gridstep; gridlow+spoint*gridstep]);
                    if  vm < valmin
                        amin = gridlow+apoint*gridstep;
                        smin = gridlow+spoint*gridstep;
                        valmin = vm;
                    end
                end
            end
        else
            amin = aguess;
            smin = sguess;
        end
        	
        z0 = [amin; smin];
        count = 0;
        while (ll(z0) == Inf) & (count <20)
            z0 = [amin; smin] .* exp(rand(2,1)*2 - 1);
            count = count + 1;
        end
        if (ll(z0)<Inf)
            options = optimset('Display','iter','MaxFunEvals', 1000, 'MaxIter', 1000);
            fval=0;
            ef=0;
            [z,fval,ef,output]=fminsearch(@ll, z0, options);
            ff = z
            sres = [sres; setupend - setupstart, output.funcCount, cputime - init, ff', 1, sest, fp, grsrch, sv, sv, Q, T, ii];
        else
            ff = [0;0]
            sres = [sres; setupend - setupstart, output.funcCount, cputime - init, ff', 1, sest, fp, grsrch, sv, sv, Q, T, ii];
        end
    end
end

sres
